use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
APIs
ASC
Alias
Alzabo
Alzabo's
Aristotle
Boolean
CPAN
Charles
Cloneable
Column
ColumnLike
Comparable
Comparison
DBI
DDL
DESC
DML
DROLSKY
DROLSKY's
Dave
Delete
Except
Exceptions
FK
FakeDBI
Fey
Fragment
FromSelect
Function
Groupable
HasAliasName
HasBindParams
HasLimitClause
HasOrderByClause
HasWhereClause
Insert
Internal
Intersect
IsLiteral
Join
Joinable
Lionheart
Literal
MYISAM
MakesAliasObjects
NULLs
Named
NamedObjectSet
Null
Number
OO
ORM
Oliver
Orderable
Pagaltzis
PayPal
Pearcey
Placeholder
Postgres
RDBMS
ReturnsData
Role
Rolsky
Rolsky's
Runtime
SQL
SQLite
SQLy
Schema
Select
Selectable
SetOperation
Siracusa
Siracusa's
SomeTable
String
SubgroupEnd
SubgroupStart
Table
TableLike
Term
Types
Union
Update
Where
alias's
attribute's
autarch
datetime
dbh
dbms
dbms's
deflator
deflators
distro
drolsky
fiddliness
fk
hdp
inflator
iterator's
lib
literal's
lookup
metaclass
metaclass's
multi
namespace
nullable
numification
oliver
pagaltzis
parameterized
params
resultset
rethrows
schemas
subclause
subref
subselect
subselects
testability
unblessed
unhashed
unsets
username
wildcard
